@@ -211,7 +211,7 @@ interpo_do(obj, shape_to, idxmap, val_from, missval, extrapo)
 		kf += idfc*cshf;
 		cshf *= shf[j];
 	    }
-            if (!nomiss && pt[it] == vmiss) {break;}
+            if (pt[it] == vmiss) {break;}
 	    if (nomiss || pf[kf] != vmiss){
 		pt[it] += a*pf[kf];
 	    } else {
