@@ -17,6 +17,8 @@ require "narray"
     * First derivative (2nd Order difference use three point.)
   * ((<cderiv>))
     * First derivative (center difference use two point.)
+  * ((<deriv2d>))
+    * 2nd derivative
   * ((<b_expand_linear_ext>))
     * return array extended boundaries with linear extention.
   * ((<cdiff>))
