@@ -13,6 +13,8 @@ require 'numru/derivative'
     * First derivative (2nd Order difference use three point.)
   * ((<cderiv>))
     * First derivative (using center difference method)
+  * ((<deriv2d>))
+    * 2nd derivative
 
 =module NumRu::GPhys::Derivative
 
@@ -70,6 +72,15 @@ Module functions of Derivative Operater for GPhys.
     RETURN VALUE
     * a GPhys
 
+---deriv2nd(gp, dim_or_dimname, bc=LINEAR_EXT, altcoord=nil)
+
+    2nd Derivate (({gp})) respect to (({dim})) th or (({dimname})) dimension 
+    covering non-uniform grids. Based on:
+      ( (z_{i+1}-z_{i})/(x_{i+1}-x_{i}) - (z_{i}-z_{i-1})/(x_{i}-x_{i-1}) )
+      / ((x_{i+1}-x_{i-1})/2)
+
+    See ((<cderiv>)) for usage.
+
 =end
 ############################################################
 
