@@ -439,6 +439,8 @@ module NumRu
 	  else
 	    val = val.to_na 
 	  end
+        elsif val.is_a?(NArrayMiss) && val.count_invalid == 0
+          val = val.to_na 
 	end
 	fcoef = FFTW3.fft( val, dir, *dims )
       else
@@ -454,6 +456,8 @@ module NumRu
 	  else
 	    val = val.to_na 
 	  end
+        elsif val.is_a?(NArrayMiss) && val.count_invalid == 0
+          val = val.to_na 
 	end
 	fcoef = FFTW.fftw( val, dir )
       end
@@ -498,6 +502,30 @@ module NumRu
       gfc
     end
 
+    def fft_deriv(dim)
+      tp = self.data.typecode
+      fc = self.fft(false,dim)
+      wn = fc.coord(dim)
+      k = wn.val.to_type(NArray::Complex)
+      n = k.length
+      n2a = (n-1)/2
+      n2b = [n/2 + 1, n-1].min  # min to avoid error if n=2 (though meaningless)
+      kmx = k[-1]+k[1]
+      ik = NArray.complex(n)
+      ik[0..n2a] = k[0..n2a]*Complex::I
+      ik[n2b..-1] = (k[n2b..-1]-kmx) * Complex::I
+      dim.times{ik.newdim!(0)}
+      (self.rank-dim-1).times{ik.newdim!(-1)}
+      fc.replace_val(fc.val*ik)
+      deriv = fc.fft(true,dim)
+      deriv.units = deriv.units * wn.units
+      if tp >= NArray::SCOMPLEX
+        deriv
+      else
+        deriv.real
+      end
+    end
+
     def __predefined_coord_units_conversion(coord)
       case coord.units
       when Units["degree"]
@@ -766,9 +794,9 @@ if $0 == __FILE__
   include NMath
 
   # < make a GPhys from scratch >
-  vx = VArray.new( NArray.float(11).indgen! * (3*Math::PI/11) ).rename("x")
+  vx = VArray.new( NArray.float(11).indgen! * (2*Math::PI/11) ).rename("x")
   vx.units = 'km'
-  vy = VArray.new( NArray.float(8).indgen! * (3*Math::PI/8) ).rename("y")
+  vy = VArray.new( NArray.float(8).indgen! * (2*Math::PI/8) ).rename("y")
   vy.units = 'km'
   xax = Axis.new().set_pos(vx)
   #yax = Axis.new(true).set_cell_guess_bounds(vy).set_pos_to_center
@@ -880,6 +908,24 @@ if $0 == __FILE__
   gpf = gp.phase_velocity_filter(0, 2, -0.3, -0.01, 1/kconv, 1/fconv)
   GGraph::tone gpf[true,0,true],true,"color_bar"=>true
 
+  ###
+  grid = Grid.new(xax, yax)
+  a = NArray.float(vx.length, vy.length)
+  a[] = sin(2*vx.val.newdim(1)) * cos(vy.val.newdim(0))
+  v = VArray.new( a )
+  v.units = 'm/s'
+  gpz = GPhys.new(grid,v)
+
+  dim = 0
+  x = gpz.coord(dim)
+  gp_x = gpz.fft_deriv(dim)
+  p gp_x
+  dim = 1
+  gp_y = gpz.fft_deriv(dim)
+  p gp_y
+  GGraph::tone gpz,true,"color_bar"=>true
+  GGraph::tone gp_x,true,"color_bar"=>true
+  GGraph::tone gp_y,true,"color_bar"=>true
 
   DCL.grcls
 end
