@@ -477,11 +477,18 @@ module NumRu
             f2 = NArray.float(len)
             for j in 0...len  
               ## [J] () ̃[vCɂقi1DȂ̂ł܂ǁC2D̂ȂꏏɁjD̍ہCg2ictr ͒TĂȂ̂ŁCinterpo_find_loc_1D 2Dł̒T g2ibl2 𒼐ڌĂԕD
-              uxi, uyi = DCL.g2ictr(txi[j], tyi[j])
-              ids1[j] = [ [uxi.floor,0].max, uxg.length-2 ].min
-              ids2[j] = [ [uyi.floor,0].max, uyg.length-2 ].min
-              f1[j] = uxi - ids1[j]
-              f2[j] = uyi - ids2[j]
+              begin
+                uxi, uyi = DCL.g2ictr(txi[j], tyi[j])
+                ids1[j] = [ [uxi.floor,0].max, uxg.length-2 ].min
+                ids2[j] = [ [uyi.floor,0].max, uyg.length-2 ].min
+                f1[j] = uxi - ids1[j]
+                f2[j] = uyi - ids2[j]
+              rescue
+                ids1[j] = -999
+                ids2[j] = -999
+                f1[j] = 0.0
+                f2[j] = 0.0
+              end
             end
             idxmap.push( [ od[0], m[1][0][0], nil, ids1, f1] ) # mapping from 1D
             idxmap.push( [ od[1], m[1][0][0], nil, ids2, f2] ) # mapping from 1D
@@ -497,11 +504,18 @@ module NumRu
             for k in 0...leny  
               for j in 0...lenx  
                 ## [J] () ̃[vCɂق(2D)D̍ہCg2ictr ͒TĂȂ̂ŁCinterpo_find_loc_1D 2Dł̒T g2ibl2 𒼐ڌĂԕD
-                uxi, uyi = DCL.g2ictr(txi[j], tyi[k])
-                ids1[j,k] = [ [uxi.floor,0].max, uxg.length-2 ].min
-                ids2[j,k] = [ [uyi.floor,0].max, uyg.length-2 ].min
-                f1[j,k] = uxi - ids1[j,k]
-                f2[j,k] = uyi - ids2[j,k]
+                begin
+                  uxi, uyi = DCL.g2ictr(txi[j], tyi[k])
+                  ids1[j,k] = [ [uxi.floor,0].max, uxg.length-2 ].min
+                  ids2[j,k] = [ [uyi.floor,0].max, uyg.length-2 ].min
+                  f1[j,k] = uxi - ids1[j,k]
+                  f2[j,k] = uyi - ids2[j,k]
+                rescue
+                  ids1[j,k] = -999
+                  ids2[j,k] = -999
+                  f1[j] = 0.0
+                  f2[j] = 0.0
+                end
               end
             end
             idxmap.push( [ od[0], m[1][0][0], [m[1][1][0]], ids1, f1] )  # mapping from 2D
