@@ -55,7 +55,7 @@ module NumRu
 	 end
       end
       def close
-	 enddef
+	 enddef rescue nil
 	 super
       end
    end        # module NetCDFDeferred
@@ -216,9 +216,15 @@ module NumRu
       ## < redefined instance methods > ##
 
       def val
-	 mode_switched = @ary.file.enddef
+         begin
+           mode_switched = @ary.file.enddef
+         rescue
+         end
 	 v = @ary.get
-	 if mode_switched; @ary.file.redef; end
+         begin
+           if mode_switched; @ary.file.redef; end
+         rescue
+         end
 	 v
       end
 
