@@ -41,19 +41,6 @@ port = ARGV.shift
 usage if port && port.to_i.to_s != port
 usage if ARGV.length > 0
 
-class NArray
-  DUMP_SIZE_LIMIT = 5000
-  def self._load(o) to_na(*Marshal::load(o)).ntoh end
-  def _dump(limit) 
-    if size <= DUMP_SIZE_LIMIT
-      Marshal::dump([hton.to_s, typecode, *shape]) 
-    else
-      raise "size of the NArray (#{size}) is too large to dump "+
-            "(limit: #{DUMP_SIZE_LIMIT})"
-    end
-  end
-end
-
 file = NetCDF.open("../testdata/T.jan.nc")
 gp = GPhys::NetCDF_IO.open(file,"T")
 
